#include "LPC17xx.h"
#include "stdint.h"
#include "touch.h" 
#include "ili9320.h"
#include "stdlib.h"
#include "math.h"
#include "sys.h"
#include "exti.h"

Pen_Holder Pen_Point;
typedef signed long  s32;

/*
*********************************************************************************************************
* Description: 	Dealy function of us.
* Arguments  : 	nus : the data of us
* Returns    : 	None
*********************************************************************************************************
*/
void delay_us(u32 nus)
{
	while(nus--);
}
/*
*********************************************************************************************************
* Description: 	Dealy function of ms.
* Arguments  : 	nms : the data of ms
* Returns    : 	None
*********************************************************************************************************
*/
void delay_ms(u32 nms)
{
	while(nms--)
	delay_us(245);
}
/*
*********************************************************************************************************
* Description: 	Write data.
* Arguments  : 	num : the data value
* Returns    : 	None
*********************************************************************************************************
*/	   
void ADS_Write_Byte(u8 num)    
{  
	u8 count=0;
	
	for(count=0;count<8;count++)  
	{ 	  
		if(num&0x80)
		{
			TDIN(1);
		}  
		else 
		{
			TDIN(0);
		}   
		num<<=1;    
		TCLK(0);
		delay_us(1);	   	 
		TCLK(1);	
	} 			    
} 
/*
*********************************************************************************************************
* Description: 	Reading the value of ADC from 7846/7843/XPT2046/UH7843/UH7846.
* Arguments  : 	CMD  the Command value
* Returns    : 	None
*********************************************************************************************************
*/		   
u16 ADS_Read_AD(u8 CMD)	  
{ 	 
	u8 count=0; 	  
	u16 Num=0;
 
	TCLK(0);	 
	TCS(0); 
	TCLK(0);
	delay_us(10);		 
	ADS_Write_Byte(CMD);
	delay_us(13);		   	    
	TCLK(0);
	delay_us(1);		 
	for(count=0;count<16;count++)  
	{ 				  
		Num<<=1; 	 
		TCLK(0);
		delay_us(1);  	    	   
		TCLK(1);
		if(DOUT)
		Num++; 		 
	}
		 	
	Num>>=4;
	TCS(1);	 
	return(Num);   
}
/*
*********************************************************************************************************
* Description: 	Take an order for the value value.
* Arguments  : 	xy  the Command value
* Returns    : 	None
*********************************************************************************************************
*/	 
#define READ_TIMES 15 
#define LOST_VAL 5	 
u16 ADS_Read_XY(u8 xy)
{
	u16 i, j;
	u16 buf[READ_TIMES];
	u16 sum=0;
	u16 temp;
	for(i=0;i<READ_TIMES;i++)
	{				 
		buf[i]=ADS_Read_AD(xy);	    
	}				    
	for(i=0;i<READ_TIMES-1; i++)
	{
		for(j=i+1;j<READ_TIMES;j++)
		{
			if(buf[i]>buf[j])
			{
				temp=buf[i];
				buf[i]=buf[j];
				buf[j]=temp;
			}
		}
	}	  
	sum=0;
	for(i=LOST_VAL;i<READ_TIMES-LOST_VAL;i++)sum+=buf[i];
	temp=sum/(READ_TIMES-2*LOST_VAL);
	return temp;   
} 
/*
*********************************************************************************************************
* Description: 	Getting the real value of AD function.
* Arguments  : 	*x ,*y   the cale of AD value
* Returns    : 	None
*********************************************************************************************************
*/	
u8 Read_ADS(u16 *x,u16 *y)
{
	u16 xtemp,ytemp;			 	 		  
	xtemp=ADS_Read_XY(CMD_RDX);
	ytemp=ADS_Read_XY(CMD_RDY);	  												   
	if(xtemp<100||ytemp<100)return 0;							//fail
	*x=xtemp;
	*y=ytemp;
	return 1;													//success
}

/*
*********************************************************************************************************
* Description: 	Getting the real value of AD function.
* Arguments  : 	x ,y  the coordinate value
* Returns    : 	None
*********************************************************************************************************
*/	
#define ERR_RANGE 50 				
u8 Read_ADS2(u16 *x,u16 *y) 
{
	u16 x1,y1;
 	u16 x2,y2;
 	u8 flag;    
    flag=Read_ADS(&x1,&y1);   
    if(flag==0)return(0);
    flag=Read_ADS(&x2,&y2);	   
    if(flag==0)return(0);   
    if(((x2<=x1&&x1<x2+ERR_RANGE)||(x1<=x2&&x2<x1+ERR_RANGE))
    &&((y2<=y1&&y1<y2+ERR_RANGE)||(y1<=y2&&y2<y1+ERR_RANGE)))
    {
        *x=(x1+x2)/2;
        *y=(y1+y2)/2;
        return 1;
    }else return 0;	  
} 
/*
*********************************************************************************************************
* Description: 	Getting the touch value of coordinate function.
* Arguments  :  None
* Returns    : 	None
*********************************************************************************************************
*/				   
u8 Read_TP_Once(void)
{
	u8 t=0;	    
	Pen_Int_Set(0);
	Pen_Point.Key_Sta=Key_Up;
	Read_ADS2(&Pen_Point.X,&Pen_Point.Y);
	while(PEN==0&&t<=250)
	{
		t++;
		delay_ms(10);
	};
	Pen_Int_Set(1);		 
	if(t>=250)return 0;
	else return 1;	
}


/*
*********************************************************************************************************
* Description: 	Getting the value of coordinate function.
* Arguments  : 	None
* Returns    : 	None
*********************************************************************************************************
*/
void Convert_Pos(void)
{		 	  
	if(Read_ADS2(&Pen_Point.X,&Pen_Point.Y))
	{
		Pen_Point.X0=Pen_Point.xfac*Pen_Point.X+Pen_Point.xoff;
		Pen_Point.Y0=Pen_Point.yfac*Pen_Point.Y+Pen_Point.yoff;  
 
	}
}
/*
*********************************************************************************************************
* Description: 	Touch screen External interrupt initialize function.
* Arguments  : 	None
* Returns    : 	None
*********************************************************************************************************
*/
uint32_t EINTInit( void )
{

  LPC_PINCON->PINSEL4 = 1 << 26;	// set P2.13 as EINT0 and
									// P2.0~7 GPIO output

  LPC_SC->EXTMODE = EINT3_EDGE;		// INT3 edge trigger
  LPC_SC->EXTPOLAR = 0;				// INT3 is falling edge by default

  NVIC_EnableIRQ(EINT3_IRQn);
  return( 1 );
}
/*
*********************************************************************************************************
* Description: 	Touch screen External interrupt function.
* Arguments  : 	None
* Returns    : 	None
*********************************************************************************************************
*/
void EINT3_IRQHandler (void) 
{
	delay_us(1);
	if(!PEN)
	{
 	Pen_Point.Key_Sta=Key_Down;
	}	

    LPC_SC->EXTINT = EINT3;				// clear interrupt
}
/*
*********************************************************************************************************
* Description: 	Touch screen External interrput Enable function.
* Arguments  : 	1 Enable
*				0 Disable
* Returns    : 	None
*********************************************************************************************************
*/	 
void Pen_Int_Set(u8 en)
{
	if(en)
	NVIC_EnableIRQ(EINT3_IRQn);   //Enable		  	
	else 
	NVIC_DisableIRQ(EINT3_IRQn); //Disable 	   
}




u16 TPReadX(void)
{ 
   u16 x=0;
   x=ADS_Read_XY(0x90);
   return (x);
}

u16 TPReadY(void)
{
 u16 y=0;
 y=ADS_Read_XY(0xD0);
 return (y);
}


void  BSP_Init(void)
{
	LPC_GPIO0->FIODIR |= 0x00002C0;  		//P0.6 0.7 0.9 Output
	LPC_GPIO2->FIODIR |= 0x0000000;  		//P2.13 input P2.0...P2.7 Output		 
	EINTInit();	 							//Setting External interrupt
	lcd_clear(0xffff);										    
}
