/*----------------------------------------------------------------------------
 * Name:    GLCD.c
 * Purpose: MCB1700 low level Graphic LCD (320x240 pixels) functions
 * Version: V1.00
 * Note(s):
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2009 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------
 * History:
 *          V1.00 Initial Version
 *----------------------------------------------------------------------------*/
//#include <LPC17xx.H>


#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include "LPC17xx.h"                    /* NXP LPC17xx definitions            */
#include "Font_24x16.h"
#include "ili9320.h"
#include "font.h"

unsigned int  HDP=799;
unsigned int  HT=1000;
unsigned int  HPS=51;
unsigned int  LPS=3;
unsigned char HPW=8;

unsigned int  VDP=479;
unsigned int  VT=530;
unsigned int  VPS=24;
unsigned int  FPS=23;
unsigned char VPW=3;
/*********************** Hardware specific configuration **********************/

/*------------------------- Speed dependant settings -------------------------*/

/* If processor works on high frequency delay has to be increased, it can be 
   increased by factor 2^N by this constant                                   */
#define DELAY_2N    18

/*---------------------- Graphic LCD size definitions ------------------------*/

#define WIDTH       800                 /* Screen Width (in pixels)           */
#define HEIGHT      480                 /* Screen Hight (in pixels)           */
#define BPP         16                  /* Bits per pixel                     */
#define BYPP        ((BPP+7)/8)         /* Bytes per pixel                    */

#define PIN_EN		(1 << 19)
#define PIN_LE		(1 << 20)
#define PIN_DIR		(1 << 21)
#define PIN_CS      (1 << 22)
#define PIN_RS		(1 << 23)
#define PIN_WR		(1 << 24)
#define PIN_RD		(1 << 25)   

/*--------------- Graphic LCD interface hardware definitions -----------------*/

/* Pin EN setting to 0 or 1                                                   */
#define LCD_EN(x)   ((x) ? (LPC_GPIO0->FIOSET = PIN_EN) : (LPC_GPIO0->FIOCLR = PIN_EN));
/* Pin LE setting to 0 or 1                                                   */
#define LCD_LE(x)   ((x) ? (LPC_GPIO0->FIOSET = PIN_LE) : (LPC_GPIO0->FIOCLR = PIN_LE));
/* Pin DIR setting to 0 or 1                                                   */
#define LCD_DIR(x)   ((x) ? (LPC_GPIO0->FIOSET = PIN_DIR) : (LPC_GPIO0->FIOCLR = PIN_DIR));
/* Pin CS setting to 0 or 1                                                   */
#define LCD_CS(x)   ((x) ? (LPC_GPIO0->FIOSET = PIN_CS) : (LPC_GPIO0->FIOCLR = PIN_CS));
/* Pin RS setting to 0 or 1                                                   */
#define LCD_RS(x)   ((x) ? (LPC_GPIO0->FIOSET = PIN_RS) : (LPC_GPIO0->FIOCLR = PIN_RS));
/* Pin WR setting to 0 or 1                                                   */
#define LCD_WR(x)   ((x) ? (LPC_GPIO0->FIOSET = PIN_WR) : (LPC_GPIO0->FIOCLR = PIN_WR));
/* Pin RD setting to 0 or 1                                                   */
#define LCD_RD(x)   ((x) ? (LPC_GPIO0->FIOSET = PIN_RD) : (LPC_GPIO0->FIOCLR = PIN_RD));
 
/*---------------------------- Global variables ------------------------------*/



/******************************************************************************/
static volatile unsigned long TextColor = BLACK, BackColor = WHITE;


/************************ Local auxiliary functions ***************************/

/*******************************************************************************
* Delay in while loop cycles                                                   *
*   Parameter:    cnt:    number of while cycles to delay                      *
*   Return:                                                                    *
*******************************************************************************/

static void delay (unsigned long cnt) {

  cnt <<= DELAY_2N;
  while (cnt--);
}

__asm void wait()
{
    nop
    BX lr
}

void wait_delay(int count)
{
  while(count--);
}

/*******************************************************************************
* Send 1 byte over serial communication                                        *
*   Parameter:    byte:   byte to be sent                                      *
*   Return:                                                                    *
*******************************************************************************/

static __inline unsigned char lcd_send (unsigned short byte) {

  LPC_GPIO2->FIODIR |= 0x000000ff;  //P2.0...P2.7 Output
  LCD_DIR(1)		   				//Interface A->B
  LCD_EN(0)	                        //Enable 2A->2B
  LPC_GPIO2->FIOPIN =  byte;        //Write D0..D7
  LCD_LE(1)                         
  LCD_LE(0)							//latch D0..D7
  LPC_GPIO2->FIOPIN =  byte >> 8;   //Write D8..D15 

  return(1);
}


/*******************************************************************************
* read 1 byte over serial communication                                        *
*   Parameter:    byte:   byte to be sent                                      *
*   Return:                                                                    *
*******************************************************************************/

static __inline unsigned short lcd_read (void) {
  unsigned short id;
  LPC_GPIO2->FIODIR &= 0xffffff00;                //P2.0...P2.7 Input
  LCD_DIR(0)		   				              //Interface B->A
  LCD_EN(0)	                                      //Enable 2B->2A
  wait_delay(10);							      //delay some times
  id = LPC_GPIO2->FIOPIN & 0x00ff;                //Read D8..D15                         
  LCD_EN(1)	                                      //Enable 1B->1A
  wait_delay(10);							      //delay some times
  id = (id << 8) | (LPC_GPIO2->FIOPIN & 0x00ff);  //Read D0..D7                         
  LCD_DIR(1)						
  return(id); 
}
 static __inline void wr_dat_start (void) {

  LCD_RS(1)
}


/*******************************************************************************
* Stop of data writing to LCD controller                                       *
*   Parameter:                                                                 *
*   Return:                                                                    *
*******************************************************************************/

static __inline void wr_dat_stop (void) {

  LCD_CS(1)
}


/*******************************************************************************
* Data writing to LCD controller                                               *
*   Parameter:    c:      data to be written                                   *
*   Return:                                                                    *
*******************************************************************************/

static __inline void wr_dat_only (unsigned short c) {

  lcd_send(c);
  LCD_WR(0)
  wait();
  LCD_WR(1)
}

/*GUI*************************************************************************************************/
static __inline void wr_cmd (unsigned char c) {
  LCD_CS(0)
  LCD_RS(0)

  LCD_RD(1)
  lcd_send(c);
  LCD_WR(0)
//  wait();
  LCD_WR(1)
  LCD_CS(1)
}

/*******************************************************************************
* Write data to LCD controller                                                 *
*   Parameter:    c:      data to be written                                   *
*   Return:                                                                    *
*******************************************************************************/


static __inline void wr_dat (unsigned short c) {

  LCD_CS(0)
  LCD_RS(1)
  LCD_RD(1)
  lcd_send(c);
  LCD_WR(0)
//  wait();
  LCD_WR(1)
  LCD_CS(1)
}
/**ILI9320GUI***************************************************************************************************************/
/*******************************************************************************
* Write command to LCD controller                                              *
*   Parameter:    c:      command to be written                                *
*   Return:                                                                    *
*******************************************************************************/

static __inline void ili9320_WriteIndex (unsigned char c) {

  LCD_CS(0)
  LCD_RS(0)

  LCD_RD(1)
  lcd_send(c);
  LCD_WR(0)
//  wait();
  LCD_WR(1)
  LCD_CS(1)
}
/*******************************************************************************
* Write data to LCD controller                                                 *
*   Parameter:    c:      data to be written                                   *
*   Return:                                                                    *
*******************************************************************************/

static __inline void ili9320_WriteData (unsigned short c) {

  LCD_CS(0)
  LCD_RS(1)
  LCD_RD(1)
  lcd_send(c);
  LCD_WR(0)
  wait();
  LCD_WR(1)
  LCD_CS(1)
}
/*******************************************************************************
* Read data from LCD controller                                                *
*   Parameter:                                                                 *
*   Return:               read data                                            *
*******************************************************************************/

static __inline unsigned short ili9320_ReadData (void) {
  unsigned short val = 0;
  LCD_CS(0)
  LCD_RS(1)
  LCD_WR(1)
  LCD_RD(0)
  val = lcd_read();
  LCD_RD(1)
  LCD_CS(1)
  return val;
}
u16 LCD_ReadRAM(void)
{
	u16 dummy;
	/* Write 16-bit Index (then Read Reg) */
	wr_cmd(0x002e); /* Select GRAM Reg */
	
	/* Read 16-bit Reg */
	dummy = ili9320_ReadData();
	dummy = ili9320_ReadData();	 
	return dummy;	
}
/*******************************************************************************
* Write to LCD register                                                        *
*   Parameter:    reg:    register to be read                                  *
*                 val:    value to write to register                           *
*******************************************************************************/

static __inline void ili9320_WriteRegister (unsigned char reg, unsigned short val) {

  LCD_CS(0)
  ili9320_WriteIndex(reg);
  ili9320_WriteData(val);
  LCD_CS(1)
}
/*******************************************************************************
* Read from LCD register                                                       *
*   Parameter:    reg:    register to be read                                  *
*   Return:               value read from register                             *
*******************************************************************************/

//static unsigned short ili9320_ReadRegister (unsigned short reg) {
//  unsigned short val = 0;
//
//  LCD_CS(0)
//  ili9320_WriteIndex(reg);
//  val = ili9320_ReadData();
//  LCD_CS(1)
//  return (val);
//}
void lcd_clear(unsigned long Color)
{
    unsigned int l=480,w;
	unsigned char rcode,gcode,bcode;

	rcode = (unsigned char)(Color >> 16);
	gcode = (unsigned char)(Color >> 8);
	bcode = (unsigned char)Color;

	 wr_cmd(0x002a);	
	wr_dat(0);	    
	wr_dat(0);
	wr_dat(HDP>>8);	    
	wr_dat(HDP&0x00ff);
     wr_cmd(0x002b);	
	wr_dat(0);	    
	wr_dat(0);
	wr_dat(VDP>>8);	    
	wr_dat(VDP&0x00ff);
	 wr_cmd(0x002c);
	
	
	while(l--)
	{
	    for(w = 0;w < HDP + 1;w++)
		{		
			wr_dat((rcode << 8) | (gcode));
			wr_dat((bcode << 8) | (rcode));
			wr_dat((gcode << 8) | (bcode));
//			wr_dat(Color);
		}
	} 
}
//static unsigned short deviceid=0;//һ̬LCDID
 uint16_t DeviceCode;
void lcd_Initializtion(void)
{

//    lcd_port_init();
  /* Configure the LCD Control pins                                           */
    LPC_GPIO2->FIODIR   |= 0x000000FF;
    LPC_GPIO0->FIODIR   |= 0x03f80000;					/*P0.19--P0.25           		*/
    LPC_GPIO0->FIOSET    = 0x03f80000;					/*P0.19--P0.25 Ϊ		*/

    delay(10);
	wr_cmd(0x00E2);	 //PLL multiplier, set PLL clock to 120M
	wr_dat(0x0023);	 //M=35
	wr_dat(0x0002);	 //N=2
	wr_dat(0x0004);	 //F=10 * (35+1) / (N+1) = 120M
//	wr_dat(0x0029);	 //M=41
//	wr_dat(0x0002);	 //N=2
//	wr_dat(0x0004);	 //F=10 * (35+1) / (N+1) = 240M
	
	wr_cmd(0x00E0);  // PLL enable
	wr_dat(0x0001);
	delay(5);
	wr_cmd(0x00E0);
	wr_dat(0x0003);
	delay(5);
	wr_cmd(0x0001);  // software reset õʱ

	delay(5);
	wr_cmd(0x00E6);	//PLL setting for PCLK, depends on resolution
	wr_dat(0x0004);
	wr_dat(0x0093);
	wr_dat(0x00e0);

	wr_cmd(0x00B0);	//LCD SPECIFICATION

//	wr_dat(0x0000);
	wr_dat(0x0020);

	wr_dat(0x0000);
	wr_dat((HDP>>8)&0X00FF);  //Set HDP
	wr_dat(HDP&0X00FF);
    wr_dat((VDP>>8)&0X00FF);  //Set VDP
	wr_dat(VDP&0X00FF);
    wr_dat(0x0000);

	wr_cmd(0x00B4);	//HSYNC
	wr_dat((HT>>8)&0X00FF);  //Set HT
	wr_dat(HT&0X00FF);
	wr_dat((HPS>>8)&0X00FF);  //Set HPS
	wr_dat(HPS&0X00FF);
	wr_dat(HPW);			   //Set HPW
	wr_dat((LPS>>8)&0X00FF);  //Set HPS
	wr_dat(LPS&0X00FF);
	wr_dat(0x0000);

	wr_cmd(0x00B6);	//VSYNC
	wr_dat((VT>>8)&0X00FF);   //Set VT
	wr_dat(VT&0X00FF);
	wr_dat((VPS>>8)&0X00FF);  //Set VPS
	wr_dat(VPS&0X00FF);
	wr_dat(VPW);			   //Set VPW
	wr_dat((FPS>>8)&0X00FF);  //Set FPS
	wr_dat(FPS&0X00FF);

	delay(5);

   	wr_cmd(0x00BA);
//	wr_dat(0x0005);    //GPIO[3:0] out 1
	wr_dat(0x000f);

	wr_cmd(0xB8);	//GPIO
//  wr_dat(0x0007);	//GPIO is controlled by host; 4IOó
	wr_dat(0x000f);
	wr_dat(0x0001);	//GPIO0 ʹIO

   	wr_cmd(0x0036);       //rotation
	wr_dat(0x0008);

	wr_cmd(0x00F0); //pixel data interface
//	wr_dat(0x0003);
	wr_dat(0x0002);

	delay(5);

	wr_cmd(0x00BC);//Ҫ
	wr_dat(0x0080);//Աȶ
	wr_dat(0x0080);//
	wr_dat(0x0080);//Ͷֵ  //
	wr_dat(0x0001);//

	delay(5);

	wr_cmd(0x00BE);        //set PWM for B/L
	wr_dat(0x0006);
	wr_dat(0x0080);
	wr_dat(0x0001);
	wr_dat(0x00f0);
	wr_dat(0x0000);
	wr_dat(0x0000);

	delay(5);

    wr_cmd(0x0021);	//ͼɫתģʽ
	wr_cmd(0x0029); //display on


//	wr_cmd(0x00d0);//ö̬ 
//	wr_dat(0x000d);

	lcd_clear(RED);
}
/****************************************************************************
*     ƣu16 ili9320_BGR2RGB(u16 c)
*     ܣRRRRRGGGGGGBBBBB Ϊ BBBBBGGGGGGRRRRR ʽ
* ڲc      BRG ɫֵ
* ڲRGB ɫֵ
* ˵    ڲ
* ÷
****************************************************************************/

uint16_t ili9320_BGR2RGB(uint16_t c)
{
  uint16_t  r, g, b, rgb;

  b = (c>>0)  & 0x1f;
  g = (c>>5)  & 0x3f;
  r = (c>>11) & 0x1f;
  
  rgb =  (b<<11) + (g<<5) + (r<<0);

  return( rgb );
}
/****************************************************************************
*     ƣvoid ili9320_SetCursor(u16 x,u16 y)
*     ܣĻ
* ڲx      
*           y      
* ڲ
* ˵    
* ÷ili9320_SetCursor(10,10);
****************************************************************************/
__inline void ili9320_SetCursor(uint16_t x,uint16_t y)

{
	if(DeviceCode==0x8989)
	{
	 	ili9320_WriteRegister(0x004e,y);        //
    	ili9320_WriteRegister(0x004f,0x13f-x);  //
	}
	else if(DeviceCode==0x9919)
	{
		ili9320_WriteRegister(0x004e,x); // 
  		ili9320_WriteRegister(0x004f,y); // 	
	}
	else
	{
  		ili9320_WriteRegister(0x0020,y); // 
  		ili9320_WriteRegister(0x0021,0x13f-x); // 
	}
}
/****************************************************************************
*     ƣu16 ili9320_GetPoint(u16 x,u16 y)
*     ܣȡָɫֵ
* ڲx      
*           y      
* ڲǰɫֵ
* ˵    
* ÷i=ili9320_GetPoint(10,10);
****************************************************************************/
uint16_t ili9320_GetPoint(uint16_t x,uint16_t y)
{
	u16 temp;
    wr_cmd(0x002A);	
	wr_dat(x>>8);	    
	wr_dat(x&0x00ff);
	wr_dat(479>>8);	    
	wr_dat(479&0x00ff);
    wr_cmd(0x002b);	
	wr_dat(y>>8);	    
	wr_dat(y&0x00ff);
	wr_dat(271>>8);	    
	wr_dat(271&0x00ff);
	wr_cmd(0x002c);
	temp = LCD_ReadRAM();

	return temp; 
}
/****************************************************************************
*     ƣvoid ili9320_SetPoint(u16 x,u16 y,u16 point)
*     ܣָ껭
* ڲx      
*           y      
*           point  ɫ
* ڲ
* ˵    
* ÷ili9320_SetPoint(10,10,0x0fe0);
****************************************************************************/

void ili9320_SetPoint(uint16_t x,uint16_t y,uint16_t point)
{
    wr_cmd(0x002A);	
	wr_dat(x>>8);	    
	wr_dat(x&0x00ff);
	wr_dat(479>>8);	    
	wr_dat(479&0x00ff);
    wr_cmd(0x002b);	
	wr_dat(y>>8);	    
	wr_dat(y&0x00ff);
	wr_dat(271>>8);	    
	wr_dat(271&0x00ff);
	wr_cmd(0x002c);
  
 	wr_dat(point);
}
/****************************************************************************
*     ƣvoid ili9320_PutChar(u16 x,u16 y,u8 c,u16 charColor,u16 bkColor)
*     ܣָʾһ8x16asciiַ
* ڲx          
*           y          
*           charColor  ַɫ
*           bkColor    ַɫ
* ڲ
* ˵    ʾΧ޶Ϊʾascii
* ÷ili9320_PutChar(10,10,'a',0x0000,0xffff);
****************************************************************************/

void ili9320_PutChar(uint16_t x,uint16_t y,uint8_t c,uint16_t charColor,uint16_t bkColor)
{
  uint16_t i=0;
  uint16_t j=0;
  
  uint16_t tmp_char=0;

  for (i=0;i<13;i++)
  {
    tmp_char=ascii_8x16[((c-0x20)*16)+i];
    for (j=0;j<8;j++)
    {
      if ( (tmp_char >> 7-j) & 0x01 == 0x01)
        {
          ili9320_SetPoint(x+j,y+i,charColor); // ַɫ
        }
        else
        {
          ili9320_SetPoint(x+j,y+i,bkColor); // ɫ
        }
    }
  }
}
/****************************************************************************
*     ƣvoid ili9320_Test()
*     ܣҺ
* ڲ
* ڲ
* ˵    ʾҺǷ
* ÷ili9320_Test();
****************************************************************************/
void ili9320_Test()
{
  uint8_t  R_data,G_data,B_data,i,j;

	ili9320_SetCursor(0x00, 0x0000);
	ili9320_WriteRegister(0x0050,0x00);//ˮƽ GRAMʼλ
	ili9320_WriteRegister(0x0051,239);//ˮƽGRAMֹλ
	ili9320_WriteRegister(0x0052,0);//ֱGRAMʼλ
	ili9320_WriteRegister(0x0053,319);//ֱGRAMֹλ  
	  LCD_CS(0);; 
	ili9320_WriteIndex(0x0022);
    R_data=0;G_data=0;B_data=0;     
    for(j=0;j<50;j++)//ɫǿ
    {
        for(i=0;i<240;i++)
            {R_data=i/8;ili9320_WriteData(R_data<<11|G_data<<5|B_data);
		}
    }
    R_data=0x1f;G_data=0x3f;B_data=0x1f;
    for(j=0;j<50;j++)
    {
        for(i=0;i<240;i++)
            {
            G_data=0x3f-(i/4);
            B_data=0x1f-(i/8);
            ili9320_WriteData(R_data<<11|G_data<<5|B_data);
			}
    }
//----------------------------------
    R_data=0;G_data=0;B_data=0;
    for(j=0;j<50;j++)//ɫǿ
    {
        for(i=0;i<240;i++)
            {G_data=i/4;
			ili9320_WriteData(R_data<<11|G_data<<5|B_data);
}
    }

    R_data=0x1f;G_data=0x3f;B_data=0x1f;
    for(j=0;j<50;j++)
    {
        for(i=0;i<240;i++)
            {
            R_data=0x1f-(i/8);
            B_data=0x1f-(i/8);
            ili9320_WriteData(R_data<<11|G_data<<5|B_data);
		}
    }
//----------------------------------
 
    R_data=0;G_data=0;B_data=0;
    for(j=0;j<60;j++)//ɫǿ
    {
        for(i=0;i<240;i++)
            {B_data=i/8;ili9320_WriteData(R_data<<11|G_data<<5|B_data);
			}
    } 

    B_data=0; 
    R_data=0x1f;G_data=0x3f;B_data=0x1f;

    for(j=0;j<60;j++)
    {
        for(i=0;i<240;i++)
            {
            G_data=0x3f-(i/4);
            R_data=0x1f-(i/8);
            ili9320_WriteData(R_data<<11|G_data<<5|B_data);
		}
    }	  
	  LCD_CS(1);;
}
/****************************************************************************
*     ƣvoid ili9320_Clear(u16 dat)
*     ܣĻָɫ 0xffff
* ڲdat      ֵ
* ڲ
* ˵    
* ÷ili9320_Clear(0xffff);
****************************************************************************/
void ili9320_Clear(uint16_t dat)
{
  uint32_t  i;

  ili9320_SetCursor(0x0000, 0x0000);  
   LCD_CS(0);
  ili9320_WriteIndex(0x0022);    
  for(i=0;i<76800;i++)
  {
    ili9320_WriteData(dat);
  }
  LCD_CS(1);
}
/****************************************************************************
*     ƣvoid ili9320_SetWindows(u16 StartX,u16 StartY,u16 EndX,u16 EndY)
*     ܣô
* ڲStartX     ʼ
*           StartY     ʼ
*           EndX       н
*           EndY       н
* ڲ
* ˵    
* ÷ili9320_SetWindows(0,0,100,100)
****************************************************************************/
__inline void ili9320_SetWindows(uint16_t StartX,uint16_t StartY,uint16_t EndX,uint16_t EndY)
{
  ili9320_SetCursor(StartX,StartY);
  ili9320_WriteRegister(0x0050, StartX);
  ili9320_WriteRegister(0x0052, StartY);
  ili9320_WriteRegister(0x0051, EndX);
  ili9320_WriteRegister(0x0053, EndY);
}
/****************************************************************************
*     ƣvoid ili9320_DrawPicture(u16 StartX,u16 StartY,u16 EndX,u16 EndY,u16 *pic)
*     ܣָ귶ΧʾһͼƬ
* ڲStartX     ʼ
*           StartY     ʼ
*           EndX       н
*           EndY       н
            pic        ͼƬͷָ
* ڲ
* ˵    ͼƬȡģʽΪˮƽɨ裬16λɫģʽ
* ÷ili9320_DrawPicture(0,0,100,100,(u16*)demo);
****************************************************************************/
void ili9320_DrawPicture(uint16_t StartX,uint16_t StartY,uint16_t EndX,uint16_t EndY,uint16_t *pic)
{
  uint16_t  i;
  ili9320_SetWindows(StartX,StartY,EndX,EndY);
  ili9320_SetCursor(StartX,StartY);
  
    LCD_CS(0);;

  ili9320_WriteIndex(0x0022);
  for (i=0;i<(EndX*EndY);i++)
  {
      ili9320_WriteData(*pic++);
  }     
    LCD_CS(1);;
}



/****************************************************************************
*     ƣvoid ili9320_Delay(vu32 nCount)
*     ܣʱ
* ڲnCount   ʱֵ
* ڲ
* ˵    
* ÷ili9320_Delay(10000);
****************************************************************************/
void ili9320_Delay(volatile uint32_t nCount)
{
  for(; nCount != 0; nCount--);
}
/*
*********************************************************************************************************
* Description: 	DrawPoint
* Arguments  : 	(x ,y) The coordinate
* Returns    : 	None
*********************************************************************************************************
*/


void LCD_DrawPoint (u16 x,u16 y ) 
{
  if ( (x>320)||(y>240) ) return;
  ili9320_SetCursor(x,y);

    LCD_CS(0);
  ili9320_WriteIndex(0x0022);
  ili9320_WriteData(BLUE);
   LCD_CS(1);
}
void Draw_Big_Point(u8 x,u16 y)
{	    
	LCD_DrawPoint(x,y);
	LCD_DrawPoint(x+1,y);
	LCD_DrawPoint(x,y+1);
	LCD_DrawPoint(x+1,y+1);	 	  	
}

/*
*********************************************************************************************************
* Description: 	Draw a touch point function.
* Arguments  : 	x ,y  the coordinate value
* Returns    : 	None
*********************************************************************************************************
*/
void Drow_Touch_Point(u8 x,u16 y)
{
	LCD_DrawPoint(x+1,y+1);
	LCD_DrawPoint(x-1,y+1);
	LCD_DrawPoint(x+1,y-1);
	LCD_DrawPoint(x-1,y-1);
}




/*******************************************************************************
* Read data from LCD controller                                                *
*   Parameter:                                                                 *
*   Return:               read data                                            *
*******************************************************************************/

static __inline unsigned short rd_dat (void) {
  unsigned short val = 0;

  LCD_RS(1)
  LCD_WR(1)
  LCD_RD(0)
  val = lcd_read();
  LCD_RD(1)
  return val;
}





static __inline void wr_reg (unsigned char reg, unsigned short val) {

  LCD_CS(0)
  wr_cmd(reg);
  wr_dat(val);
  LCD_CS(1)
}

void GLCD_WindowMax (void) {

  wr_reg(0x50, 0);                      /* Horizontal GRAM Start Address      */
  wr_reg(0x51, HEIGHT-1);               /* Horizontal GRAM End   Address (-1) */
  wr_reg(0x52, 0);                      /* Vertical   GRAM Start Address      */
  wr_reg(0x53, WIDTH-1);                /* Vertical   GRAM End   Address (-1) */
}

static unsigned short rd_reg (unsigned short reg) {
  unsigned short val = 0;

  LCD_CS(0)
  wr_cmd(reg);
  val = rd_dat(); 
  LCD_CS(1)
  return (val);
}


void GLCD_init (void) { 
  static unsigned short driverCode;
	LPC_GPIO2->FIODIR0 = 0xff; 
 
  /* Configure the LCD Control pins                                           */
  LPC_GPIO0->FIODIR   |= 0x03f80000;
  LPC_GPIO0->FIOSET    = 0x03f80000;	

  delay(5);                             /* Delay 50 ms                        */

  driverCode = rd_reg(0x00);

  if(driverCode == 0x4531)				//2.8" TFT LCD Module,DriverIC is LGDP4531
  {
  	wr_reg(0x00,0x0001);
  	wr_reg(0x10,0x0628);
  	wr_reg(0x12,0x0006);
  	wr_reg(0x13,0x0A32);
  	wr_reg(0x11,0x0040);
  	wr_reg(0x15,0x0050);
  	wr_reg(0x12,0x0016);
  	delay(15);
  	wr_reg(0x10,0x5660);
  	delay(15);
  	wr_reg(0x13,0x2A4E);
  	wr_reg(0x01,0x0100);
  	wr_reg(0x02,0x0300);
	
  	wr_reg(0x03,0x1030);
	
  	wr_reg(0x08,0x0202);
  	wr_reg(0x0A,0x0000);
  	wr_reg(0x30,0x0000);
  	wr_reg(0x31,0x0402);
  	wr_reg(0x32,0x0106);
  	wr_reg(0x33,0x0700);
  	wr_reg(0x34,0x0104);
  	wr_reg(0x35,0x0301);
  	wr_reg(0x36,0x0707);
  	wr_reg(0x37,0x0305);
  	wr_reg(0x38,0x0208);
  	wr_reg(0x39,0x0F0B);
  	delay(15);
  	wr_reg(0x41,0x0002);
  	wr_reg(0x60,0x2700);
  	wr_reg(0x61,0x0001);
  	wr_reg(0x90,0x0119);
  	wr_reg(0x92,0x010A);
  	wr_reg(0x93,0x0004);
  	wr_reg(0xA0,0x0100);
  	delay(15);
  	wr_reg(0xA0,0x0000);
  	delay(20);
  }
  else if(driverCode == 0x9320)		 //3.2" TFT LCD Module,DriverIC is ILI9320
  {
    /* Start Initial Sequence --------------------------------------------------*/
	wr_reg(0xE5, 0x8000);                 /* Set the internal vcore voltage     */
	wr_reg(0x00, 0x0001);                 /* Start internal OSC                 */
	wr_reg(0x01, 0x0100);                 /* Set SS and SM bit                  */
	wr_reg(0x02, 0x0700);                 /* Set 1 line inversion               */
	wr_reg(0x03, 0x1030);                 /* Set GRAM write direction and BGR=1 */
	wr_reg(0x04, 0x0000);                 /* Resize register                    */
	wr_reg(0x08, 0x0202);                 /* 2 lines each, back and front porch */
	wr_reg(0x09, 0x0000);                 /* Set non-disp area refresh cyc ISC  */
	wr_reg(0x0A, 0x0000);                 /* FMARK function                     */
	wr_reg(0x0C, 0x0000);                 /* RGB interface setting              */
	wr_reg(0x0D, 0x0000);                 /* Frame marker Position              */
	wr_reg(0x0F, 0x0000);                 /* RGB interface polarity             */
	
	/* Power On sequence -------------------------------------------------------*/
	wr_reg(0x10, 0x0000);                 /* Reset Power Control 1              */
	wr_reg(0x11, 0x0000);                 /* Reset Power Control 2              */
	wr_reg(0x12, 0x0000);                 /* Reset Power Control 3              */
	wr_reg(0x13, 0x0000);                 /* Reset Power Control 4              */
	delay(20);                            /* Discharge cap power voltage (200ms)*/
    wr_reg(0x10, 0x17B0);                 /* SAP, BT[3:0], AP, DSTB, SLP, STB   */
	wr_reg(0x11, 0x0137);                 /* DC1[2:0], DC0[2:0], VC[2:0]        */
	delay(5);                             /* Delay 50 ms                        */
	wr_reg(0x12, 0x0139);                 /* VREG1OUT voltage                   */
	delay(5);                             /* Delay 50 ms                        */
	wr_reg(0x13, 0x1D00);                 /* VDV[4:0] for VCOM amplitude        */
	wr_reg(0x29, 0x0013);                 /* VCM[4:0] for VCOMH                 */
	delay(5);                             /* Delay 50 ms                        */
	wr_reg(0x20, 0x0000);                 /* GRAM horizontal Address            */
	wr_reg(0x21, 0x0000);                 /* GRAM Vertical Address              */
	
	/* Adjust the Gamma Curve --------------------------------------------------*/
	wr_reg(0x30, 0x0006);
	wr_reg(0x31, 0x0101);
	wr_reg(0x32, 0x0003);
	wr_reg(0x35, 0x0106);
	wr_reg(0x36, 0x0B02);
	wr_reg(0x37, 0x0302);
	wr_reg(0x38, 0x0707);
	wr_reg(0x39, 0x0007);
	wr_reg(0x3C, 0x0600);
	wr_reg(0x3D, 0x020B);
	  
	/* Set GRAM area -----------------------------------------------------------*/
	wr_reg(0x50, 0x0000);                 /* Horizontal GRAM Start Address      */
	wr_reg(0x51, (HEIGHT-1));             /* Horizontal GRAM End   Address      */
	wr_reg(0x52, 0x0000);                 /* Vertical   GRAM Start Address      */
	wr_reg(0x53, (WIDTH-1));              /* Vertical   GRAM End   Address      */
	wr_reg(0x60, 0x2700);                 /* Gate Scan Line                     */
	wr_reg(0x61, 0x0001);                 /* NDL,VLE, REV                       */
	wr_reg(0x6A, 0x0000);                 /* Set scrolling line                 */
	
	/* Partial Display Control -------------------------------------------------*/
	wr_reg(0x80, 0x0000);
	wr_reg(0x81, 0x0000);
	wr_reg(0x82, 0x0000);
	wr_reg(0x83, 0x0000);
	wr_reg(0x84, 0x0000);
	wr_reg(0x85, 0x0000);
	
	/* Panel Control -----------------------------------------------------------*/
	wr_reg(0x90, 0x0010);
	wr_reg(0x92, 0x0000);
	wr_reg(0x93, 0x0003);
	wr_reg(0x95, 0x0110);
	wr_reg(0x97, 0x0000);
	wr_reg(0x98, 0x0000);
  }
  /* Set GRAM write direction and BGR = 1
     I/D=10 (Horizontal : increment, Vertical : increment)
     AM=1 (address is updated in vertical writing direction)                  */
  wr_reg(0x03, 0x1038);

  wr_reg(0x07, 0x0173);                 /* 262K color and display ON          */ 
}

void GLCD_PutPixel (unsigned int x, unsigned int y) {

  wr_reg(0x20, y);
  wr_reg(0x21, WIDTH-1-x);
  LCD_CS(0)
  wr_cmd(0x22);
  wr_dat(TextColor);
  LCD_CS(1)
}

/*******************************************************************************
* Set foreground color                                                         *
*   Parameter:    color:  color for clearing display                           *
*   Return:                                                                    *
*******************************************************************************/
void GLCD_setTextColor(unsigned short color) {
  TextColor = color;
}


/*******************************************************************************
* Set background color                                                         *
*   Parameter:    color:  color for clearing display                           *
*   Return:                                                                    *
*******************************************************************************/
void GLCD_setBackColor(unsigned short color) {
  BackColor = color;
}


/*******************************************************************************
* Clear display                                                                *
*   Parameter:    color:  color for clearing display                           *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_clear (unsigned short color) {
  unsigned int   i;

  GLCD_WindowMax();

  wr_reg(0x20, 0);
  wr_reg(0x21, 0);
  LCD_CS(0)
  wr_cmd(0x22);
  wr_dat_start();
  for(i = 0; i < (WIDTH*HEIGHT); i++)
    wr_dat_only(color);
  wr_dat_stop();
}


/*******************************************************************************
* Draw character on given position                                             *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*                   c:        pointer to character bitmap                      *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_DrawChar (unsigned int x, unsigned int y, unsigned short *c) {
  int idx = 0, i, j;

  x = WIDTH-x-CHAR_W;
  wr_reg(0x50, y);                      /* Horizontal GRAM Start Address      */
  wr_reg(0x51, y+CHAR_H-1);             /* Horizontal GRAM End   Address (-1) */
  wr_reg(0x52, x);                      /* Vertical   GRAM Start Address      */
  wr_reg(0x53, x+CHAR_W-1);             /* Vertical   GRAM End   Address (-1) */

  wr_reg(0x20, y);
  wr_reg(0x21, x);
  LCD_CS(0)
  wr_cmd(0x22);
  wr_dat_start();
  for (j = 0; j < CHAR_H; j++) {
    for (i = CHAR_W-1; i >= 0; i--) {
      if((c[idx] & (1 << i)) == 0x00) {
        wr_dat_only(BackColor);
      } else {
        wr_dat_only(TextColor);
      }
    }
    c++;
  }
  wr_dat_stop();
}


/*******************************************************************************
* Disply character on given line                                               *
*   Parameter:      ln:       line number                                      *
*                   col:      column number                                    *
*                   c:        ascii character                                  *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_DisplayChar (unsigned int ln, unsigned int col, unsigned char c) {

  c -= 32;
  GLCD_DrawChar(col * CHAR_W, ln * CHAR_H, (unsigned short *)&Font_24x16[c * CHAR_H]);
}


/*******************************************************************************
* Disply string on given line                                                  *
*   Parameter:      ln:       line number                                      *
*                   col:      column number                                    *
*                   s:        pointer to string                                *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_DisplayString (unsigned int ln, unsigned int col, unsigned char *s) {

  GLCD_WindowMax();
  while (*s) {
    GLCD_DisplayChar(ln, col++, *s++);
  	if(col*CHAR_W > WIDTH){
		ln ++;
		col = 0;
	}
  }
}

//void GLCD_displayStringLn(unsigned int ln, unsigned char *s);
void GLCD_displayStringLn (unsigned int ln, unsigned char *s) {
  int col;
  col = 0;
  GLCD_WindowMax();
  while (*s) {
    GLCD_DisplayChar(ln, col++, *s++);
  }
}

/*******************************************************************************
* Clear given line                                                             *
*   Parameter:      ln:       line number                                      *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_ClearLn (unsigned int ln) {

  GLCD_WindowMax();
  GLCD_DisplayString(ln, 0, "                    ");
}

/*******************************************************************************
* Draw bargraph                                                                *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*                   w:        maximum width of bargraph (in pixels)            *
*                   val:      value of active bargraph (in 1/1024)             *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_Bargraph (unsigned int x, unsigned int y, unsigned int w, unsigned int h, unsigned int val) {
  int i,j;

  x = WIDTH-x-w;
  wr_reg(0x50, y);                      /* Horizontal GRAM Start Address      */
  wr_reg(0x51, y+CHAR_H-1);             /* Horizontal GRAM End   Address (-1) */
  wr_reg(0x52, x);                      /* Vertical   GRAM Start Address      */
  wr_reg(0x53, x+w-1);                  /* Vertical   GRAM End   Address (-1) */

  val = (val * w) >> 10;                /* Scale value for 24x12 characters   */
  wr_reg(0x20, y);
  wr_reg(0x21, x);
  LCD_CS(0)
  wr_cmd(0x22);
  wr_dat_start();
  for (i = 0; i < h; i++) {
    for (j = w-1; j >= 0; j--) {
      if(j >= val) {
        wr_dat_only(BackColor);
      } else {
        wr_dat_only(TextColor);
      }
    }
  }
  wr_dat_stop();
}


/*******************************************************************************
* Display graphical bitmap image at position x horizontally and y vertically   *
* (This function is optimized for 16 bits per pixel format, it has to be       *
*  adapted for any other bits per pixel format)                                *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*                   w:        width of bitmap                                  *
*                   h:        height of bitmap                                 *
*                   bitmap:   address at which the bitmap data resides         *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_Bitmap (unsigned int x, unsigned int y, unsigned int w, unsigned int h, unsigned char *bitmap) {
  unsigned int    i, j;
  unsigned short *bitmap_ptr = (unsigned short *)bitmap;

  x = WIDTH-x-w;
  wr_reg(0x50, y);                      /* Horizontal GRAM Start Address      */
  wr_reg(0x51, y+h-1);                  /* Horizontal GRAM End   Address (-1) */
  wr_reg(0x52, x);                      /* Vertical   GRAM Start Address      */
  wr_reg(0x53, x+w-1);                  /* Vertical   GRAM End   Address (-1) */

  wr_reg(0x20, y);
  wr_reg(0x21, x);
  LCD_CS(0)
  wr_cmd(0x22);
  wr_dat_start();
  for (j = 0; j < h; j++) {
    bitmap_ptr += w-1;
    for (i = 0; i < w; i++) {
      wr_dat_only(*bitmap_ptr--);
    }
    bitmap_ptr += w+1;
  }
  wr_dat_stop();
}


/*******************************************************************************
* Display graphical bmp file image at position x horizontally and y vertically *
* (This function is optimized for 16 bits per pixel format, it has to be       *
*  adapted for any other bits per pixel format)                                *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*                   w:        width of bitmap                                  *
*                   h:        height of bitmap                                 *
*                   bmp:      address at which the bmp data resides            *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_Bmp (unsigned int x, unsigned int y, unsigned int w, unsigned int h, unsigned char *bmp) {
  unsigned int    i, j;
  unsigned short *bitmap_ptr = (unsigned short *)bmp;

  x = WIDTH-x-w;
  wr_reg(0x50, y);                      /* Horizontal GRAM Start Address      */
  wr_reg(0x51, y+h-1);                  /* Horizontal GRAM End   Address (-1) */
  wr_reg(0x52, x);                      /* Vertical   GRAM Start Address      */
  wr_reg(0x53, x+w-1);                  /* Vertical   GRAM End   Address (-1) */

  wr_reg(0x20, y);
  wr_reg(0x21, x);
  LCD_CS(0)
  wr_cmd(0x22);
  wr_dat_start();
  bitmap_ptr += (h*w)-1;
  for (j = 0; j < h; j++) {
    for (i = 0; i < w; i++) {
      wr_dat_only(*bitmap_ptr--);
    }
  }
  wr_dat_stop();
}





#define XTAL        (12000000UL)        /* Oscillator frequency               */
#define OSC_CLK     (      XTAL)        /* Main oscillator frequency          */
#define RTC_CLK     (   32000UL)        /* RTC oscillator frequency           */
#define IRC_OSC     ( 4000000UL)        /* Internal RC oscillator frequency   */
#define CLOCK_SETUP           1
#define SCS_Val               0x00000020
#define CLKSRCSEL_Val         0x00000001
#define PLL0_SETUP            1
#define PLL0CFG_Val           0x00050063
#define PLL1_SETUP            0
#define PLL1CFG_Val           0x00000023
#define CCLKCFG_Val           0x00000003
#define USBCLKCFG_Val         0x00000000
#define PCLKSEL0_Val          0x00000000
#define PCLKSEL1_Val          0x00000000
#define PCONP_Val             0x042887DE
#define CLKOUTCFG_Val         0x00000000
uint32_t SystemFrequency = IRC_OSC;
//void SystemInit (void)
//{
//#if (CLOCK_SETUP)                       /* Clock Setup                        */
//  LPC_SC->SCS       = SCS_Val;
//  if (SCS_Val & (1 << 5)) {             /* If Main Oscillator is enabled      */
//    while ((LPC_SC->SCS & (1<<6)) == 0);/* Wait for Oscillator to be ready    */
//  }
//
//  LPC_SC->CCLKCFG   = CCLKCFG_Val;      /* Setup Clock Divider                */
//
//  LPC_SC->PCLKSEL0  = PCLKSEL0_Val;     /* Peripheral Clock Selection         */
//  LPC_SC->PCLKSEL1  = PCLKSEL1_Val;
//
//  LPC_SC->CLKSRCSEL = CLKSRCSEL_Val;    /* Select Clock Source for PLL0       */
//
//#if (PLL0_SETUP)
//  LPC_SC->PLL0CFG   = PLL0CFG_Val;
//  LPC_SC->PLL0CON   = 0x01;             /* PLL0 Enable                        */
//  LPC_SC->PLL0FEED  = 0xAA;
//  LPC_SC->PLL0FEED  = 0x55;
//  while (!(LPC_SC->PLL0STAT & (1<<26)));/* Wait for PLOCK0                    */
//
//  LPC_SC->PLL0CON   = 0x03;             /* PLL0 Enable & Connect              */
//  LPC_SC->PLL0FEED  = 0xAA;
//  LPC_SC->PLL0FEED  = 0x55;
//#endif
//
//#if (PLL1_SETUP)
//  LPC_SC->PLL1CFG   = PLL1CFG_Val;
//  LPC_SC->PLL1CON   = 0x01;             /* PLL1 Enable                        */
//  LPC_SC->PLL1FEED  = 0xAA;
//  LPC_SC->PLL1FEED  = 0x55;
//  while (!(LPC_SC->PLL1STAT & (1<<10)));/* Wait for PLOCK1                    */
//
//  LPC_SC->PLL1CON   = 0x03;             /* PLL1 Enable & Connect              */
//  LPC_SC->PLL1FEED  = 0xAA;
//  LPC_SC->PLL1FEED  = 0x55;
//#else
//  LPC_SC->USBCLKCFG = USBCLKCFG_Val;    /* Setup USB Clock Divider            */
//#endif
//
//  LPC_SC->PCONP     = PCONP_Val;        /* Power Control for Peripherals      */
//
//  LPC_SC->CLKOUTCFG = CLKOUTCFG_Val;    /* Clock Output Configuration         */
//#endif
//
//  /* Determine clock frequency according to clock register values             */
//  if (((LPC_SC->PLL0STAT >> 24)&3)==3) {/* If PLL0 enabled and connected      */
//    switch (LPC_SC->CLKSRCSEL & 0x03) {
//      case 0:                           /* Internal RC oscillator => PLL0     */
//      case 3:                           /* Reserved, default to Internal RC   */
//        SystemFrequency = (IRC_OSC * 
//                          ((2 * ((LPC_SC->PLL0STAT & 0x7FFF) + 1)))  /
//                          (((LPC_SC->PLL0STAT >> 16) & 0xFF) + 1)    /
//                          ((LPC_SC->CCLKCFG & 0xFF)+ 1));
//        break;
//      case 1:                           /* Main oscillator => PLL0            */
//        SystemFrequency = (OSC_CLK * 
//                          ((2 * ((LPC_SC->PLL0STAT & 0x7FFF) + 1)))  /
//                          (((LPC_SC->PLL0STAT >> 16) & 0xFF) + 1)    /
//                          ((LPC_SC->CCLKCFG & 0xFF)+ 1));
//        break;
//      case 2:                           /* RTC oscillator => PLL0             */
//        SystemFrequency = (RTC_CLK * 
//                          ((2 * ((LPC_SC->PLL0STAT & 0x7FFF) + 1)))  /
//                          (((LPC_SC->PLL0STAT >> 16) & 0xFF) + 1)    /
//                          ((LPC_SC->CCLKCFG & 0xFF)+ 1));
//        break;
//    }
//  } else {
//    switch (LPC_SC->CLKSRCSEL & 0x03) {
//      case 0:                           /* Internal RC oscillator => PLL0     */
//      case 3:                           /* Reserved, default to Internal RC   */
//        SystemFrequency = IRC_OSC / ((LPC_SC->CCLKCFG & 0xFF)+ 1);
//        break;
//      case 1:                           /* Main oscillator => PLL0            */
//        SystemFrequency = OSC_CLK / ((LPC_SC->CCLKCFG & 0xFF)+ 1);
//        break;
//      case 2:                           /* RTC oscillator => PLL0             */
//        SystemFrequency = RTC_CLK / ((LPC_SC->CCLKCFG & 0xFF)+ 1);
//        break;
//    }
//  }
//
//#if (FLASH_SETUP == 1)                  /* Flash Accelerator Setup            */
//  LPC_SC->FLASHCFG  = (LPC_SC->FLASHCFG & ~0x0000F000) | FLASHCFG_Val;
//#endif
//}


void system_wait(void){
	ili9320_Test();
	do{

    }while((LPC_GPIO2->FIOPIN1 >> 2) & 1);
}
