/*----------------------------------------------------------------------------
 * Name:    GLCD.h
 * Purpose: MCB1700 low level Graphic LCD (320x240 pixels) prototypes
 * Version: V1.00
 * Note(s):
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2009 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------
 * History:
 *          V1.00 Initial Version
 *----------------------------------------------------------------------------*/

#ifndef _ILI9320_H
#define _ILI9320_H
#include "stdint.h"
#include "sys.h"
/*------------------------------------------------------------------------------
  color coding.
  GLCD is coded:   15..11 red, 10..5 green, 4..0 blue  (unsigned short)  GLCD_R5, GLCD_G6, GLCD_B5   
  original coding: 17..12 red, 11..6 green, 5..0 blue                    ORG_R6,  ORG_G6,  ORG_B6

  ORG_R1..5 = GLCD_R0..4,  ORG_R0 = GLCD_R4
  ORG_G0..5 = GLCD_G0..5,
  ORG_B1..5 = GLCD_B0..4,  ORG_B0 = GLCD_B4
 *------------------------------------------------------------------------------*/
#define WHITE         	 0xFFFFFF
#define BLACK         	 0x000000	  
#define BLUE         	 0x0000FF  
#define RED           	 0xFF0000
#define MAGENTA       	 0xCD00CD
#define GREEN         	 0x00FF00
#define CYAN          	 0x00FFFF
#define YELLOW        	 0xFFFF00
#define BROWN 			 0X8B2323 //ɫ
#define BRRED 			 0X8B1A1A //غɫ
#define GRAY  			 0XE6E6FA //ɫ
#define LGRAY 			 0XF0F8FF //ǳɫ                            

#define Line0                0
#define Line1               24
#define Line2               48
#define Line3               72
#define Line4               96
#define Line5              120
#define Line6              144
#define Line7              168
#define Line8              192
#define Line9              216


extern void GLCD_init           (void);
extern void GLCD_clear          (unsigned short color);
extern void GLCD_setTextColor   (unsigned short color);
extern void GLCD_setBackColor   (unsigned short color);
extern void GLCD_displayChar    (unsigned int ln, unsigned int col, unsigned char  c);
extern void GLCD_displayStringLn(unsigned int ln, unsigned char *s);
extern void GLCD_Bitmap         (unsigned int x, unsigned int y, unsigned int w, unsigned int h, unsigned char *bitmap);
extern void GLCD_clearLn        (unsigned int ln);
extern void GLCD_putPixel       (unsigned int x, unsigned int y);
/*ILI9320GUI*****************************************************************************/
void lcd_Initializtion(void);
void lcd_clear(unsigned long Color);
void ili9320_SetPoint(uint16_t x,uint16_t y,uint16_t point);
uint16_t ili9320_GetPoint(uint16_t x,uint16_t y);
void Drow_Touch_Point(u8 x,u16 y);


#endif /* _GLCD_H */
